<?php
/**
 * Enhancements to the WordPress backend.
 *
 * @package Tasty_Pins
 */

namespace Tasty_Pins;

use Tasty_Pins\Converters\Mediavine_Grow;
use Tasty_Pins\Converters\Social_Warfare;

/**
 * Enhancements to the WordPress backend.
 */
class Admin {
	/**
	 * Plugin About page slug.
	 *
	 * @var string
	 */
	const ABOUT_SLUG = 'tasty-pins-about';

	/**
	 * Capability required to manage settings.
	 *
	 * @var string
	 */
	const CAPABILITY = 'edit_others_posts';

	/**
	 * Post meta key used to store the default text.
	 *
	 * @var string
	 */
	const DEFAULT_TEXT_KEY = 'tp_pinterest_default_text';

	/**
	 * Post meta key used to store the default title.
	 *
	 * @var string
	 */
	const DEFAULT_TITLE_KEY = 'tp_pinterest_default_title';

	/**
	 * Post meta key used to store Pinterest Text.
	 *
	 * @var string
	 */
	const TEXT_KEY = 'tp_pinterest_text';

	/**
	 * Post meta key used to store Pinterest Title.
	 *
	 * @var string
	 */
	const TITLE_KEY = 'tp_pinterest_title';

	/**
	 * Post meta key used to store Pinterest Repin ID.
	 *
	 * @var string
	 */
	const REPIN_ID_KEY = 'tp_pinterest_repin_id';

	/**
	 * Post meta key used to store "Force pinning" option.
	 *
	 * @var string
	 */
	const FORCE_PINNING_KEY = 'tp_pinterest_force_pinning';

	/**
	 * Post meta key used to store Pinterest hidden image.
	 *
	 * @var string
	 */
	const HIDDEN_IMAGE_KEY = 'tp_pinterest_hidden_image';

	/**
	 * Name of the plugin author.
	 *
	 * @var string
	 */
	const ITEM_AUTHOR = 'Tasty Pins';

	/**
	 * Name of the product.
	 *
	 * @var string
	 */
	const ITEM_NAME = 'Tasty Pins';

	/**
	 * Key used for the license check cache.
	 *
	 * @var string
	 */
	const LICENSE_CHECK_CACHE_KEY = 'tasty-pins-license-check';

	/**
	 * Option used to store the license key.
	 *
	 * @var string
	 */
	const LICENSE_KEY_OPTION = 'tasty_pins_license_key';

	/**
	 * Key used for the settings nonce.
	 *
	 * @var string
	 */
	const NONCE_KEY = 'tasty-pins-settings';

	/**
	 * Slug for the settings page.
	 *
	 * @var string
	 */
	const PAGE_SLUG = 'tasty-pins';

	/**
	 * Option name for plugin activation state.
	 *
	 * @var string
	 */
	const PLUGIN_ACTIVATION_OPTION = 'tasty_pins_do_activation_redirect';

	/**
	 * Group used for configuration settings.
	 *
	 * @var string
	 */
	const SETTINGS_GROUP_CONFIG = 'tasty-pins-settings-config';

	/**
	 * Name for the Tasty Pins settings.
	 *
	 * @var string
	 */
	const SETTINGS_OPTION = 'tasty_pins_settings';

	/**
	 * URL for the EDD store.
	 *
	 * @var
	 */
	const STORE_URL = 'https://www.wptasty.com';

	/**
	 * Initialize the upgrader
	 */
	public static function action_admin_init() {
		new Updater(
			self::STORE_URL,
			TASTY_PINS_PLUGIN_FILE,
			array(
				'version'   => TASTY_PINS_PLUGIN_VERSION,
				'license'   => get_option( self::LICENSE_KEY_OPTION ),
				'item_name' => self::ITEM_NAME,
				'author'    => self::ITEM_AUTHOR,
			)
		);

		if ( get_option( self::PLUGIN_ACTIVATION_OPTION, false ) ) {
			delete_option( self::PLUGIN_ACTIVATION_OPTION );
			if ( ! isset( $_GET['activate-multi'] ) ) {
				wp_safe_redirect( add_query_arg( 'tab', 'about', menu_page_url( self::PAGE_SLUG, false ) ) );
				exit;
			}
		}
	}

	/**
	 * Actions to perform when activating the plugin.
	 */
	public static function plugin_activation() {
		update_option( self::PLUGIN_ACTIVATION_OPTION, true );
	}

	/**
	 * Includes PHP and plugin versions in the user agent for update checks.
	 *
	 * @param array  $r   An array of HTTP request arguments.
	 * @param string $url The request URL.
	 * @return array
	 */
	public static function filter_http_request_args( $r, $url ) {
		if ( self::STORE_URL !== $url
			|| 'POST' !== $r['method']
			|| empty( $r['body'] )
			|| ! is_array( $r['body'] )
			|| empty( $r['body']['item_name'] )
			|| self::ITEM_NAME !== $r['body']['item_name'] ) {
			return $r;
		}

		$r['user-agent'] = rtrim( $r['user-agent'], ';' )
			. '; PHP/' . PHP_VERSION . '; '
			. self::ITEM_NAME . '/' . TASTY_PINS_PLUGIN_VERSION;
		return $r;
	}

	/**
	 * Register admin menu UX.
	 */
	public static function action_admin_menu() {
		register_setting( 'tasty-pins-settings', self::LICENSE_KEY_OPTION );
		$option_name = self::SETTINGS_OPTION;
		register_setting(
			self::SETTINGS_GROUP_CONFIG,
			$option_name,
			array(
				'sanitize_callback' => function( $val ) use ( $option_name ) {
					$existing = get_option( $option_name );
					$retval   = array();
					foreach (
						array(
							'hover_button_type'           => 'sanitize_title',
							'hover_button_position'       => 'sanitize_title',
							'hover_button_shape'          => 'sanitize_title',
							'image_overlay_enabled'       => 'sanitize_title',
							'custom_button_label_enabled' => 'sanitize_title',
							'custom_button_label_text'    => 'sanitize_text_field',
							'minimum_image_width'         => 'absint',
							'minimum_image_height'        => 'absint',
						)
						as $key_name => $sanitize_callback ) {
						if ( isset( $val[ $key_name ] ) ) {
							$retval[ $key_name ] = $sanitize_callback( $val[ $key_name ] );
						} elseif ( isset( $existing[ $key_name ] ) ) {
							$retval[ $key_name ] = $existing[ $key_name ];
						}
						if ( in_array( $key_name, array( 'minimum_image_width', 'minimum_image_height' ), true ) && empty( $retval[ $key_name ] ) ) {
							$retval[ $key_name ] = '';
						}
					}
					return $retval;
				},
			)
		);
		add_options_page( __( 'Tasty Pins', 'tasty-pins' ), __( 'Tasty Pins', 'tasty-pins' ), self::CAPABILITY, self::PAGE_SLUG, array( __CLASS__, 'handle_settings_page' ) );

	}

	/**
	 * Display an admin notice when the license key is incorrect.
	 */
	public static function action_admin_notices_license_key() {

		$screen = get_current_screen();
		if ( $screen && 'post' === $screen->base ) {
			return;
		}

		if ( ! get_option( self::LICENSE_KEY_OPTION ) ) :
			?>
	<div class="updated" style="display:block !important;">
		<form method="post" action="options.php">
			<p>
				<strong><?php esc_html_e( 'Tasty Pins is almost ready', 'tasty-pins' ); ?></strong>, <label style="vertical-align: baseline;" for="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>"><?php esc_html_e( 'enter your license key to continue', 'tasty-pins' ); ?></label>
				<input type="text" style="margin-left: 5px; margin-right: 5px; " class="code regular-text" id="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>" name="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>" />
				<input type="submit" value="<?php _e( 'Save license key', 'tasty-pins' ); ?>" class="button-primary" />
			</p>
			<p>
				<strong><?php esc_html_e( "Don't have a Tasty Pins license yet?", 'tasty-pins' ); ?></strong> <a href="https://www.wptasty.com/" target="_blank"><?php esc_html_e( 'Get one in just a few minutes time', 'tasty-pins' ); ?></a>, <?php esc_html_e( "and report back once you've gotten your license key", 'tasty-pins' ); ?>
			</p>
			<?php
			settings_fields( 'tasty-pins-settings' );
			do_settings_sections( 'tasty-pins-settings' );
			?>
		</form>

	</div>
			<?php
		endif;

		$license_check = false;
		if ( get_option( self::LICENSE_KEY_OPTION ) ) {
			$license_check = get_transient( self::LICENSE_CHECK_CACHE_KEY );
			if ( false === $license_check ) {
				$api_params = array(
					'edd_action' => 'check_license',
					'license'    => get_option( self::LICENSE_KEY_OPTION ),
					'item_id'    => false,
					'item_name'  => self::ITEM_NAME,
					'author'     => self::ITEM_AUTHOR,
					'url'        => home_url(),
				);

				$license_check = wp_remote_post(
					self::STORE_URL,
					array(
						'timeout' => 15,
						'body'    => $api_params,
					)
				);

				if ( ! is_wp_error( $license_check ) ) {
					$license_check = json_decode( wp_remote_retrieve_body( $license_check ) );
				}
				set_transient( self::LICENSE_CHECK_CACHE_KEY, $license_check, 60 * HOUR_IN_SECONDS );
			}
		}

		if ( ! empty( $license_check ) && 'valid' !== $license_check->license ) :
			?>
	<div class="error" style="display:block !important;">
		<form method="post" action="options.php">
			<p>
				<strong><?php esc_html_e( 'To enable updates and support for Tasty Pins', 'tasty-pins' ); ?></strong>, <label style="vertical-align: baseline;" for="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>"><?php esc_html_e( 'enter a valid license key', 'tasty-pins' ); ?></label>
				<input type="text" style="margin-left: 5px; margin-right: 5px; " class="code regular-text" id="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>" name="<?php echo esc_attr( self::LICENSE_KEY_OPTION ); ?>" value="<?php echo esc_attr( get_option( self::LICENSE_KEY_OPTION ) ); ?>" />
				<input type="submit" value="<?php _e( 'Save license key', 'tasty-pins' ); ?>" class="button-primary" />
			</p>
			<p>
				<strong><?php esc_html_e( "Think you've reached this message in error?", 'tasty-pins' ); ?></strong> <a href="http://support.wptasty.com" target="_blank"><?php esc_html_e( 'Submit a support ticket', 'tasty-pins' ); ?></a>, <?php esc_html_e( "and we'll do our best to help out.", 'tasty-pins' ); ?>
			</p>
			<div style="display:none;"><pre><?php echo json_encode( $license_check ); ?></pre></div>
			<?php
			settings_fields( 'tasty-pins-settings' );
			do_settings_sections( 'tasty-pins-settings' );
			?>
		</form>

	</div>
			<?php
		endif;

	}

	/**
	 * Activate the license when the option is updated
	 */
	public static function action_update_option_register_license() {
		$api_params = array(
			'edd_action' => 'activate_license',
			'license'    => get_option( self::LICENSE_KEY_OPTION ),
			'item_name'  => self::ITEM_NAME, // the name of our product in EDD.
			'url'        => home_url(),
		);
		wp_remote_post(
			self::STORE_URL,
			array(
				'timeout' => 15,
				'body'    => $api_params,
			)
		);
	}

	/**
	 * Clear license key cache when the option is updated.
	 */
	public static function action_update_option_clear_transient() {
		delete_transient( self::LICENSE_CHECK_CACHE_KEY );
		$cache_key = md5( 'edd_plugin_' . sanitize_key( plugin_basename( TASTY_PINS_PLUGIN_FILE ) ) . '_version_info' );
		delete_site_transient( $cache_key );
		$cache_key = 'edd_api_request_' . substr( md5( serialize( basename( TASTY_PINS_PLUGIN_FILE, '.php' ) ) ), 0, 15 );
		delete_site_transient( $cache_key );
		delete_site_transient( 'update_plugins' );
	}

	/**
	 * Include the 'Settings' and 'Remove license' links in plugin actions
	 *
	 * @param array $links Existing plugin actions.
	 * @return array
	 */
	public static function filter_plugin_action_links( $links ) {
		$links['tp_settings'] = '<a href="' . esc_url( menu_page_url( self::PAGE_SLUG, false ) ) . '">' . esc_html__( 'Settings', 'tasty-pins' ) . '</a>';
		if ( get_option( self::LICENSE_KEY_OPTION ) ) {
			$links['remove_key'] = '<a href="' . add_query_arg(
				array(
					'action' => 'tasty_pins_remove_license_key',
					'nonce'  => wp_create_nonce( 'tasty_pins_remove_license_key' ),
				),
				admin_url( 'admin-ajax.php' )
			) . '">' . esc_html__( 'Remove license', 'tasty-pins' ) . '</a>';
		}
		return $links;
	}

	/**
	 * Handle AJAX request to remove the license key.
	 */
	public static function handle_wp_ajax_remove_license_key() {

		if ( ! current_user_can( 'manage_options' ) || ! wp_verify_nonce( $_GET['nonce'], 'tasty_pins_remove_license_key' ) ) {
			wp_safe_redirect( admin_url( 'plugins.php' ) );
			exit;
		}

		$api_params = array(
			'edd_action' => 'deactivate_license',
			'license'    => get_option( self::LICENSE_KEY_OPTION ),
			'item_name'  => self::ITEM_NAME,
			'url'        => home_url(),
		);
		$response   = wp_remote_post(
			self::STORE_URL,
			array(
				'timeout' => 15,
				'body'    => $api_params,
			)
		);
		if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
			if ( is_wp_error( $response ) ) {
				$message = $response->get_error_message();
			} else {
				$message = __( 'An error occurred, please try again.', 'tasty-pins' );
			}
			wp_die( $message );
		}

		self::action_update_option_clear_transient();
		delete_option( self::LICENSE_KEY_OPTION );
		wp_safe_redirect( admin_url( 'plugins.php' ) );
		exit;
	}

	/**
	 * Handles an AJAX request to convert Social Warfare Pinterest images.
	 */
	public static function handle_wp_ajax_convert() {
		if ( ! wp_verify_nonce( $_GET['nonce'], self::NONCE_KEY )
			|| empty( $_GET['post_id'] ) ) {
			wp_send_json_error(
				array(
					'message' => __( "Sorry, you don't have permission to do this.", 'tasty-pins' ),
				)
			);
		}
		if ( 'mediavine-grow' === $_GET['type'] ) {
			$ret = Mediavine_Grow::convert_post( (int) $_GET['post_id'] );
		} elseif ( 'social-warfare' === $_GET['type'] ) {
			$ret = Social_Warfare::convert_post( (int) $_GET['post_id'] );
		} else {
			wp_send_json_error(
				array(
					'message' => __( 'Unknown converter type.', 'tasty-pins' ),
				)
			);
		}
		$response = array(
			'message' => is_wp_error( $ret ) ? $ret->get_error_message() : '',
		);
		wp_send_json_success( $response );
	}

	/**
	 * Add Pinterest Text HTML to the DOM.
	 *
	 * @param WP_Post $post   Attachment object.
	 */
	public static function action_edit_form_after_title( $post ) {
		if ( 'attachment' !== $post->post_type
			|| 'image' !== substr( $post->post_mime_type, 0, 5 ) ) {
			return;
		}
		$text_value = get_post_meta( $post->ID, self::TEXT_KEY, true );
		$text_name  = 'attachments[' . $post->ID . '][' . self::TEXT_KEY . ']';

		$title_value = get_post_meta( $post->ID, self::TITLE_KEY, true );
		$title_name  = 'attachments[' . $post->ID . '][' . self::TITLE_KEY . ']';

		$id_value = get_post_meta( $post->ID, self::REPIN_ID_KEY, true );
		$id_name  = 'attachments[' . $post->ID . '][' . self::REPIN_ID_KEY . ']';
		?>
		<p id="tp-pinterest-text-wrap" class="hide-if-js">
			<label for="<?php echo esc_attr( $text_name ); ?>"><strong>Pinterest Text</strong></label><br>
			<textarea style="height:4em;" name="<?php echo esc_attr( $text_name ); ?>" class="widefat" data-tasty-pins-character-limit="500"><?php echo esc_textarea( $text_value ); ?></textarea>
		</p>
		<p id="tp-pinterest-title-wrap" class="hide-if-js">
			<label for="<?php echo esc_attr( $title_name ); ?>"><strong>Pinterest Title</strong></label><br>
			<input type="text" name="<?php echo esc_attr( $title_name ); ?>" value="<?php echo esc_attr( $title_value ); ?>" class="widefat" data-tasty-pins-character-limit="70">
		</p>
		<p id="tp-pinterest-repin-id-wrap" class="hide-if-js">
			<label for="<?php echo esc_attr( $id_name ); ?>"><strong>Pinterest Repin ID</strong></label><br>
			<input type="text" name="<?php echo esc_attr( $id_name ); ?>" value="<?php echo esc_attr( $id_value ); ?>" class="widefat">
		</p>
		<?php
	}

	/**
	 * Add a meta box to manage the hidden Pinterest image
	 *
	 * @param string $post_type Post type being loaded.
	 */
	public static function action_add_meta_boxes( $post_type ) {
		if ( 'attachment' === $post_type
			|| ! post_type_supports( $post_type, 'editor' ) ) {
			return;
		}
		add_meta_box( 'tasty-pins', esc_html__( 'Tasty Pins', 'tasty-pins' ), array( __CLASS__, 'render_meta_box' ), $post_type );
	}

	/**
	 * Render the Tasty Pins meta box
	 *
	 * @param WP_Post $post Current post object.
	 */
	public static function render_meta_box( $post ) {
		$force_pinning = (bool) get_post_meta( $post->ID, self::FORCE_PINNING_KEY, true );
		$default_text  = get_post_meta( $post->ID, self::DEFAULT_TEXT_KEY, true );
		$default_title = get_post_meta( $post->ID, self::DEFAULT_TITLE_KEY, true );
		$hidden_images = get_post_meta( $post->ID, self::HIDDEN_IMAGE_KEY, true );
		$hidden_images = array_map( 'intval', explode( ',', $hidden_images ) );
		?>
		<section>
			<h3><?php esc_html_e( 'Pinterest hidden images', 'tasty-pins' ); ?></h3>
			<div class="tasty-pins-force-pinning">
				<label>
					<input type="checkbox" <?php checked( true, $force_pinning ); ?> name="<?php echo esc_attr( self::FORCE_PINNING_KEY ); ?>">
					<?php esc_html_e( 'Force pinning of hidden images', 'tasty-pins' ); ?>
				</label>
			</div>
			<div class="tasty-pins-hidden-images">
			<div class="tasty-pins-hidden-image tasty-pins-hidden-image-with-placeholder">
				<div class="tasty-pins-hidden-image-preview">
					<div class="tasty-pins-hidden-image-preview-placeholder"></div>
				</div>
				<span class="tasty-pins-select-hidden-image button" data-input-name="<?php echo esc_attr( self::HIDDEN_IMAGE_KEY ); ?>" data-media-title="<?php esc_attr_e( 'Pinterest Hidden Images', 'tasty-pins' ); ?>"><?php esc_html_e( 'Select Images', 'tasty-pins' ); ?></span>
			</div>
			<?php
			foreach ( $hidden_images as $hidden_image ) {
				$hidden_image_src = wp_get_attachment_image_src( $hidden_image, 'thumbnail' );
				$hidden_image_src = ! empty( $hidden_image_src[0] ) ? $hidden_image_src[0] : false;
				if ( empty( $hidden_image_src ) ) {
					continue;
				}
				self::render_single_image_template( $hidden_image_src, $hidden_image );
			}
			?>
			</div>
		</section>
		<div style="clear:both;"></div>
		<section class="tasty-pins-default-title">
			<h3 for="<?php echo esc_attr( self::DEFAULT_TITLE_KEY ); ?>">
				<?php esc_html_e( 'Default Pinterest Title', 'tasty-pins' ); ?>
			</h3>
			<p><?php esc_html_e( 'Default Pinterest titles will be applied to any images in the content that do not have one, including hidden images. Note: Pinterest is still in the process of rolling out support for Pinterest titles. Currently, Pinterest is automatically pulling Pinterest Titles from your post titles.', 'tasty-pins' ); ?> <a target="_blank" href="<?php echo esc_url( 'https://support.wptasty.com/en/articles/1334388-what-should-i-put-in-my-alt-text-title-text-and-pinterest-text' ); ?>"><?php esc_html_e( 'Learn more.', 'tasty-pins' ); ?></a></p>
			<p><?php esc_html_e( 'You have up to 70 characters for your Pin title.', 'tasty-pins' ); ?></p>
			<input name="<?php echo esc_attr( self::DEFAULT_TITLE_KEY ); ?>" style="display:inline-block;margin-top: 0.5rem;" value="<?php echo esc_attr( $default_title ); ?>" data-tasty-pins-character-limit="70" />
		</section>
		<section class="tasty-pins-default-text">
			<h3 for="<?php echo esc_attr( self::DEFAULT_TEXT_KEY ); ?>">
				<?php esc_html_e( 'Default Pinterest Text', 'tasty-pins' ); ?>
			</h3>
			<p><?php esc_html_e( 'Default Pinterest text will be applied to any images in the content that do not have Pinterest text, including hidden images.', 'tasty-pins' ); ?> <a target="_blank" href="<?php echo esc_url( 'https://support.wptasty.com/en/articles/2913815-how-does-default-pinterest-text-work' ); ?>"><?php esc_html_e( 'Learn more.', 'tasty-pins' ); ?></a></p>
			<p><?php esc_html_e( 'You have up to 500 characters for your Pin description, but keep in mind only the first 30-60 characters will be visible in the feed description.', 'tasty-pins' ); ?></p>
			<textarea name="<?php echo esc_attr( self::DEFAULT_TEXT_KEY ); ?>" rows="4" style="margin-top: 0.5rem;" data-tasty-pins-character-limit="500"><?php echo esc_textarea( $default_text ); ?></textarea>
		</section>
		<script type="text/html" id="tmpl-tasty-pins-hidden-image-preview">
			<?php self::render_single_image_template( '{{data.src}}', '{{data.id}}' ); ?>
		</script>
		<?php
		wp_enqueue_media();
		wp_nonce_field( 'tasty-pins-nonce', 'tasty-pins-nonce' );
	}

	/**
	 * Add a meta box to manage the hidden Pinterest image.
	 *
	 * @param object $term Term being displayed.
	 */
	public static function action_category_edit_form( $term ) {
		$default_text  = get_term_meta( $term->term_id, self::DEFAULT_TEXT_KEY, true );
		$default_title = get_term_meta( $term->term_id, self::DEFAULT_TITLE_KEY, true );
		$hidden_images = get_term_meta( $term->term_id, self::HIDDEN_IMAGE_KEY, true );
		$hidden_images = array_map( 'intval', explode( ',', $hidden_images ) );
		?>
		<div class="postbox tasty-pins-metabox">
			<h2>Tasty Pins</h2>
			<div class="inside">
				<section>
					<h3><?php esc_html_e( 'Pinterest hidden images', 'tasty-pins' ); ?></h3>
					<div class="tasty-pins-hidden-images">
					<div class="tasty-pins-hidden-image tasty-pins-hidden-image-with-placeholder">
						<div class="tasty-pins-hidden-image-preview">
							<div class="tasty-pins-hidden-image-preview-placeholder"></div>
						</div>
						<span class="tasty-pins-select-hidden-image button" data-input-name="<?php echo esc_attr( self::HIDDEN_IMAGE_KEY ); ?>" data-media-title="<?php esc_attr_e( 'Pinterest Hidden Images', 'tasty-pins' ); ?>"><?php esc_html_e( 'Select Images', 'tasty-pins' ); ?></span>
					</div>
					<?php
					foreach ( $hidden_images as $hidden_image ) {
						$hidden_image_src = wp_get_attachment_image_src( $hidden_image, 'thumbnail' );
						$hidden_image_src = ! empty( $hidden_image_src[0] ) ? $hidden_image_src[0] : false;
						if ( empty( $hidden_image_src ) ) {
							continue;
						}
						self::render_single_image_template( $hidden_image_src, $hidden_image );
					}
					?>
					</div>
				</section>
				<div style="clear:both;"></div>
				<section class="tasty-pins-default-title">
					<h3 for="<?php echo esc_attr( self::DEFAULT_TITLE_KEY ); ?>">
						<?php esc_html_e( 'Default Pinterest Title', 'tasty-pins' ); ?>
					</h3>
					<p><?php esc_html_e( 'Default Pinterest titles will be applied to any hidden images that do not have one. Note: Pinterest is still in the process of rolling out support for Pinterest titles. Currently, Pinterest is automatically pulling Pinterest Titles from your post titles.', 'tasty-pins' ); ?> <a target="_blank" href="<?php echo esc_url( 'https://support.wptasty.com/en/articles/1334388-what-should-i-put-in-my-alt-text-title-text-and-pinterest-text' ); ?>"><?php esc_html_e( 'Learn more.', 'tasty-pins' ); ?></a></p>
					<p><?php esc_html_e( 'You have up to 70 characters for your Pin title.', 'tasty-pins' ); ?></p>
					<input name="<?php echo esc_attr( self::DEFAULT_TITLE_KEY ); ?>" style="display:inline-block;margin-top: 0.5rem;" value="<?php echo esc_attr( $default_title ); ?>" data-tasty-pins-character-limit="70" />
				</section>
				<section class="tasty-pins-default-text">
					<h3 for="<?php echo esc_attr( self::DEFAULT_TEXT_KEY ); ?>">
						<?php esc_html_e( 'Default Pinterest Text', 'tasty-pins' ); ?>
					</h3>
					<p><?php esc_html_e( 'Default Pinterest descriptions will be applied to any any hidden images that do not have Pinterest Text.', 'tasty-pins' ); ?> <a target="_blank" href="<?php echo esc_url( 'https://support.wptasty.com/en/articles/1334388-what-should-i-put-in-my-alt-text-title-text-and-pinterest-text' ); ?>"><?php esc_html_e( 'Learn more.', 'tasty-pins' ); ?></a></p>
					<p><?php esc_html_e( 'You have up to 500 characters for your Pin description, but keep in mind only the first 30-60 characters will be visible in the feed description.', 'tasty-pins' ); ?></p>
					<textarea name="<?php echo esc_attr( self::DEFAULT_TEXT_KEY ); ?>" rows="4" style="margin-top: 0.5rem;" data-tasty-pins-character-limit="500"><?php echo esc_textarea( $default_text ); ?></textarea>
				</section>
			</div>
		</div>
		<script type="text/html" id="tmpl-tasty-pins-hidden-image-preview">
			<?php self::render_single_image_template( '{{data.src}}', '{{data.id}}' ); ?>
		</script>
		<?php
		wp_enqueue_media();
		wp_nonce_field( 'tasty-pins-nonce', 'tasty-pins-nonce' );
	}

	/**
	 * Render the display of a single preview image.
	 *
	 * @param string  $src          Source for the hidden image.
	 * @param integer $hidden_image ID for the hidden image.
	 */
	private static function render_single_image_template( $src, $hidden_image ) {
		$id = md5( mt_rand() );
		?>
		<div class="tasty-pins-hidden-image">
			<div class="tasty-pins-hidden-image-preview">
				<img src="<?php echo esc_attr( $src ); ?>" data-pin-nopin="1" />
			</div>
			<span class="tasty-pins-remove-hidden-image button" data-input-id="<?php echo esc_attr( $id ); ?>"><span class="dashicons dashicons-no-alt"></span></span>
			<input type="hidden" id="<?php echo esc_attr( $id ); ?>" name="<?php echo esc_attr( self::HIDDEN_IMAGE_KEY ); ?>[]" value="<?php echo esc_attr( $hidden_image ); ?>" />
		</div>
		<?php
	}

	/**
	 * Save the meta box when the post is saved
	 *
	 * @param integer $post_id ID for the post.
	 */
	public static function action_save_post( $post_id ) {

		if ( ! isset( $_POST['tasty-pins-nonce'] )
			|| ! wp_verify_nonce( $_POST['tasty-pins-nonce'], 'tasty-pins-nonce' ) ) {
			return;
		}
		$force_pinning = ! empty( $_POST[ self::FORCE_PINNING_KEY ] ) ? 1 : 0;
		update_post_meta( $post_id, self::FORCE_PINNING_KEY, $force_pinning );

		$hidden_images = ! empty( $_POST[ self::HIDDEN_IMAGE_KEY ] ) ? implode( ',', array_map( 'intval', $_POST[ self::HIDDEN_IMAGE_KEY ] ) ) : '';
		update_post_meta( $post_id, self::HIDDEN_IMAGE_KEY, $hidden_images );

		$default_text = ! empty( $_POST[ self::DEFAULT_TEXT_KEY ] ) ? sanitize_text_field( $_POST[ self::DEFAULT_TEXT_KEY ] ) : '';
		update_post_meta( $post_id, self::DEFAULT_TEXT_KEY, $default_text );

		$default_title = ! empty( $_POST[ self::DEFAULT_TITLE_KEY ] ) ? sanitize_text_field( $_POST[ self::DEFAULT_TITLE_KEY ] ) : '';
		update_post_meta( $post_id, self::DEFAULT_TITLE_KEY, $default_title );
	}

	/**
	 * Save the meta box when the category is saved
	 *
	 * @param integer $term_id ID for the term being saved.
	 */
	public static function action_saved_category( $term_id ) {

		if ( ! isset( $_POST['tasty-pins-nonce'] )
			|| ! wp_verify_nonce( $_POST['tasty-pins-nonce'], 'tasty-pins-nonce' ) ) {
			return;
		}

		$hidden_images = ! empty( $_POST[ self::HIDDEN_IMAGE_KEY ] ) ? implode( ',', array_map( 'intval', $_POST[ self::HIDDEN_IMAGE_KEY ] ) ) : '';
		update_term_meta( $term_id, self::HIDDEN_IMAGE_KEY, $hidden_images );

		$default_text = ! empty( $_POST[ self::DEFAULT_TEXT_KEY ] ) ? sanitize_text_field( $_POST[ self::DEFAULT_TEXT_KEY ] ) : '';
		update_term_meta( $term_id, self::DEFAULT_TEXT_KEY, $default_text );

		$default_title = ! empty( $_POST[ self::DEFAULT_TITLE_KEY ] ) ? sanitize_text_field( $_POST[ self::DEFAULT_TITLE_KEY ] ) : '';
		update_term_meta( $term_id, self::DEFAULT_TITLE_KEY, $default_title );
	}

	/**
	 * Save custom fields used in the attachment editor.
	 *
	 * @param array $data       Attachment fields to save.
	 * @param array $attachment Data specific about the attachment.
	 * @return array
	 */
	public static function filter_attachment_fields_to_save( $data, $attachment ) {
		foreach ( array(
			self::TEXT_KEY,
			self::TITLE_KEY,
			self::REPIN_ID_KEY,
		) as $field ) {
			if ( ! isset( $attachment[ $field ] ) ) {
				continue;
			}
			update_post_meta( $data['ID'], $field, sanitize_text_field( $attachment[ $field ] ) );
		}
		return $data;
	}

	/**
	 * Filter the attachment data prepared for JavaScript.
	 *
	 * @param array  $response   Array of prepared attachment data.
	 * @param object $attachment Attachment object.
	 * @return array
	 */
	public static function filter_wp_prepare_attachment_for_js( $response, $attachment ) {
		foreach ( array(
			self::TEXT_KEY,
			self::TITLE_KEY,
			self::REPIN_ID_KEY,
		) as $field ) {
			$response[ $field ] = get_post_meta( $attachment->ID, $field, true );
		}
		return $response;
	}

	/**
	 * Save Pinterest Text when an attachment is saved.
	 */
	public static function handle_wp_ajax_save_attachment() {
		if ( ! isset( $_POST['id'] ) ) {
			return;
		}
		if ( ! check_ajax_referer( 'update-post_' . $_POST['id'], 'nonce' ) ) {
			return;
		}

		foreach ( array(
			self::TEXT_KEY,
			self::TITLE_KEY,
			self::REPIN_ID_KEY,
		) as $field ) {
			if ( ! isset( $_POST['changes'][ $field ] ) ) {
				continue;
			}
			update_post_meta( (int) $_POST['id'], $field, sanitize_text_field( $_POST['changes'][ $field ] ) );
		}
	}

	/**
	 * Append Pinterest Text to markup sent to editor
	 *
	 * @param string $html The image HTML markup to send.
	 * @return string
	 */
	public static function filter_image_send_to_editor( $html ) {
		if ( ! empty( $_POST['props']['tp_pinterest_text'] ) ) {
			$html = str_replace( '<img ', '<img data-pin-description="' . esc_attr( wp_unslash( $_POST['props']['tp_pinterest_text'] ) ) . '" ', $html );
		}
		if ( ! empty( $_POST['props']['tp_pinterest_title'] ) ) {
			$html = str_replace( '<img ', '<img data-pin-title="' . esc_attr( wp_unslash( $_POST['props']['tp_pinterest_title'] ) ) . '" ', $html );
		}
		if ( ! empty( $_POST['props']['tp_pinterest_repin_id'] ) ) {
			$html = str_replace( '<img ', '<img data-pin-id="' . esc_attr( wp_unslash( $_POST['props']['tp_pinterest_repin_id'] ) ) . '" ', $html );
		}
		if ( ! empty( $_POST['props']['tp_pinterest_nopin'] ) ) {
			$html = str_replace( '<img ', '<img data-pin-nopin="true" ', $html );
		}
		return $html;
	}

	/**
	 * Filter allowed tags to include our data attributes.
	 *
	 * @param array  $tags    Allowed tags.
	 * @param string $context Context for the call.
	 * @return array
	 */
	public static function filter_wp_kses_allowed_html( $tags, $context ) {
		if ( 'post' !== $context ) {
			return $tags;
		}
		$tags['img']['data-pin-description'] = true;
		$tags['img']['data-pin-title']       = true;
		$tags['img']['data-pin-id']          = true;
		$tags['img']['data-pin-nopin']       = true;
		return $tags;
	}

	/**
	 * Get a rendered template.
	 *
	 * @param string $template Template to render.
	 * @param array  $vars     Variables to pass to the template.
	 * @return string
	 */
	public static function get_template_part( $template, $vars = array() ) {
		$full_path = dirname( dirname( __FILE__ ) ) . '/templates/' . $template . '.php';
		// Provided template may already be a full path.
		if ( ! file_exists( $full_path ) ) {
			$full_path = $template;
		}
		if ( ! file_exists( $full_path ) ) {
			return '';
		}

		ob_start();
		// @codingStandardsIgnoreStart
		if ( ! empty( $vars ) ) {
			extract( $vars );
		}
		// @codingStandardsIgnoreEnd
		include $full_path;
		return ob_get_clean();
	}

	/**
	 * Render the settings page.
	 */
	public static function handle_settings_page() {
		echo self::get_template_part( 'settings' );
	}

	/**
	 * Render the About page.
	 */
	public static function handle_about_page() {
		echo self::get_template_part( 'about' );
	}

}
