<?php
/**
 * Converts Mediavine Grow data over to Tasty Pins.
 *
 * @package Tasty_Pins
 */

namespace Tasty_Pins\Converters;

use Tasty_Pins\Admin;
use WP_Error;

/**
 * Converts Mediavine Grow data over to Tasty Pins.
 */
class Mediavine_Grow {

	/**
	 * Meta key used to indicate when Mediavine Grow has already been migrated.
	 *
	 * @var string
	 */
	const CONVERSION_COMPLETE_KEY = 'tasty_pins_converted_mediavine_grow';

	/**
	 * Meta key used by Mediavine Grow to store hidden images.
	 *
	 * @var string
	 */
	const PINTEREST_HIDDEN_IMAGES_KEY = 'dpsp_pinterest_hidden_images';

	/**
	 * Meta key used by Mediavine Grow to store social options.
	 *
	 * @var string
	 */
	const PINTEREST_SHARE_OPTIONS_KEY = 'dpsp_share_options';

	/**
	 * Meta key used by Mediavine Grow to store the Pinterest Title on the attachment.
	 *
	 * @var string
	 */
	const ATTACHMENT_PINTEREST_TITLE_KEY = 'pin_title';

	/**
	 * Meta key used by Mediavine Grow to store the Pinterest Text on the attachment.
	 *
	 * @var string
	 */
	const ATTACHMENT_PINTEREST_TEXT_KEY = 'pin_description';

	/**
	 * Meta key used by Mediavine Grow to store the Repin ID on the attachment.
	 *
	 * @var string
	 */
	const ATTACHMENT_PINTEREST_REPIN_ID_KEY = 'pin_repin_id';

	/**
	 * Get the post ids that have unmigrated Social Warfare data.
	 *
	 * @return array
	 */
	public static function get_unmigrated_post_ids() {
		global $wpdb;

		$mgp_post_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT DISTINCT(post_id) FROM {$wpdb->postmeta} WHERE meta_key = %s OR meta_key = %s OR meta_key = %s OR meta_key = %s OR meta_key = %s",
				self::PINTEREST_HIDDEN_IMAGES_KEY,
				self::PINTEREST_SHARE_OPTIONS_KEY,
				self::ATTACHMENT_PINTEREST_TITLE_KEY,
				self::ATTACHMENT_PINTEREST_TEXT_KEY,
				self::ATTACHMENT_PINTEREST_REPIN_ID_KEY
			)
		);
		if ( empty( $mgp_post_ids ) ) {
			return array();
		}
		$converted_post_ids = $wpdb->get_col( $wpdb->prepare( "SELECT DISTINCT(post_id) FROM {$wpdb->postmeta} WHERE meta_key = %s", self::CONVERSION_COMPLETE_KEY ) );
		$unconverted_ids    = array_diff( $mgp_post_ids, $converted_post_ids );

		return array_values( $unconverted_ids );
	}

	/**
	 * Converts Mediavine Grow data to Tasty Pins for a post.
	 *
	 * @param integer $post_id Post ID to convert.
	 * @return true|WP_Error
	 */
	public static function convert_post( $post_id ) {

		if ( ! $post_id || ! get_post( $post_id ) ) {
			// translators: The ID for the post.
			return new WP_Error( 'tp_missing_post', sprintf( __( 'No post found for post id %d', 'tasty-pins' ), $post_id ) );
		}

		$share_options = get_post_meta( $post_id, self::PINTEREST_SHARE_OPTIONS_KEY, true );
		// Set the Pinterest title if none is set.
		if ( ! empty( $share_options['custom_title_pinterest'] )
			&& ! get_post_meta( $post_id, Admin::DEFAULT_TITLE_KEY, true ) ) {
			update_post_meta(
				$post_id,
				Admin::DEFAULT_TITLE_KEY,
				$share_options['custom_title_pinterest']
			);
		}
		// Set the Pinterest text if none is set.
		if ( ! empty( $share_options['custom_description_pinterest'] )
			&& ! get_post_meta( $post_id, Admin::DEFAULT_TEXT_KEY, true ) ) {
			update_post_meta(
				$post_id,
				Admin::DEFAULT_TEXT_KEY,
				$share_options['custom_description_pinterest']
			);
		}

		// Set the hidden images if there aren't any yet.
		if ( ! get_post_meta( $post_id, Admin::HIDDEN_IMAGE_KEY, true ) ) {
			$hidden_images = array();
			if ( ! empty( $share_options['custom_image_pinterest']['id'] ) ) {
				$hidden_images[] = $share_options['custom_image_pinterest']['id'];
			}
			$others = get_post_meta( $post_id, self::PINTEREST_HIDDEN_IMAGES_KEY, true );
			if ( ! empty( $others ) ) {
				$hidden_images = array_merge( $hidden_images, $others );
			}
			if ( ! empty( $hidden_images ) ) {
				update_post_meta(
					$post_id,
					Admin::HIDDEN_IMAGE_KEY,
					implode(
						',',
						array_map(
							'intval',
							$hidden_images
						)
					)
				);
			}
		}

		// Attachment data handling.
		$fields = array(
			self::ATTACHMENT_PINTEREST_TITLE_KEY    => Admin::TITLE_KEY,
			self::ATTACHMENT_PINTEREST_TEXT_KEY     => Admin::TEXT_KEY,
			self::ATTACHMENT_PINTEREST_REPIN_ID_KEY => Admin::REPIN_ID_KEY,
		);
		foreach ( $fields as $grow => $tp ) {
			$value = get_post_meta( $post_id, $grow, true );
			if ( ! empty( $value )
				&& ! get_post_meta( $post_id, $tp, true ) ) {
				update_post_meta( $post_id, $tp, $value );
			}
		}

		// Set a marker to prevent future conversions.
		update_post_meta( $post_id, self::CONVERSION_COMPLETE_KEY, 1 );

		return true;
	}

}
